//------------------------------------------------------------------------------
// Title        : Analog to Digital library
// Authors      : Anton Rieckert (anton@riecktron.co.za)
// Target MCU   : PIC24
// Last Updated : 29 March 2012
// Updates      : www.riecktron.co.za
//------------------------------------------------------------------------------
#include "libADC.h"
//------------------------------------------------------------------------------

unsigned int      FSampleBuffer[16];
unsigned int      FChannelCount;
void (*FADCIRQCallback) (unsigned int ANumOfSamples, unsigned int *ASamples[16]);

//------------------------------------------------------------------------------
void ADC_InterruptSampleReady(void) org 0x2E {

int   i;
unsigned int   *ADC16Ptr;

   ADC16Ptr = &ADC1BUF0;                     // Initialize ADC1BUF pointer
   
   for ( i = 0; i < FChannelCount; i++ ) {
      FSampleBuffer[i] = *ADC16Ptr;         // Insert sample into buffer
      ADC16Ptr++;                           // Next sample
   }

   FADCIRQCallback(FChannelCount, &FSampleBuffer);

  // Clear the ADC done flag
  IFS0.B13 = 0;
  
}

//------------------------------------------------------------------------------
void ADC_Enable(void) {

  // Clear ADC interrupt flag
  IFS0.B13 = 0;

  // Setup the ADC done interrupt
  IEC0.B13 = 1;
  
}

//------------------------------------------------------------------------------
void ADC_Disable(void) {

  // Setup the ADC done interrupt
  IEC0.B13 = 0;

  // Clear ADC interrupt flag
  IFS0.B13 = 0;

}

//------------------------------------------------------------------------------
void ADC_StartSample(void) {
  // Start the sampling
  AD1CON1.B2 = 1;                       // auto start sampling for 31Tad
}

//------------------------------------------------------------------------------
void ADC_Init(unsigned int Achannels,unsigned  int Afs, void (*TADCIRQCallback) (unsigned int ANumOfSamples, unsigned int *ASamples[16])) {

  unsigned int   i;
  char txt[200];
  
  unsigned long calcInterval1;
  unsigned int T3PreScale;


  // Set the sample ready callback function
  FADCIRQCallback = TADCIRQCallback;
  
  // Get the number of samples
  FChannelCount = 0;
  
  for ( i = 0; i < 16; i++ ) {
    if (((unsigned int)Achannels & (unsigned int)(1 << i)) > 0) {
      FChannelCount++;
    }
  }

  // Bit mask for the channels that are enabled
  AD1PCFG = (unsigned int)~((unsigned int)Achannels);
  
  // Internal counter triggers conversion
  AD1CON1 = 0x0040;
  
  // Not used
  AD1CHS = 0x0000;
  
  // Enable channel scanning
  AD1CSSL = Achannels;

  // Clock from system clock
  // 31 Tad
  // 64 Tcy
  AD1CON3 = 0x1F0F;

  // Set AD1IF after every ASampleCount samples
  // Remember to read ASampleCount samples in interrupt
  // Do not alternate between MUXA and MUXB
  // Scan all enabled channels on MUXA
  AD1CON2 = 0x0400 | ((FChannelCount - 1) << 2);

  // Calculate the sampling frequency to put into TMR3
  calcInterval1 = Clock_kHz() / 2;
  calcInterval1 = (calcInterval1 * 1000) / Afs;
  calcInterval1 = calcInterval1 / FChannelCount;

  T3PreScale = 0;
  if (calcInterval1 > 65535) {
    T3PreScale = 1;
    calcInterval1 = calcInterval1 / 8;
  }
  
  if (calcInterval1 > 65535) {
    T3PreScale = 2;
    calcInterval1 = calcInterval1 / 8;
  }

  if (calcInterval1 > 65535) {
    T3PreScale = 3;
    calcInterval1 = calcInterval1 / 4;
  }

  // Make use of Timer 3 for the sampling periode
  T3CON = 0x8000 | (T3PreScale << 4);
  PR3 = calcInterval1;

  AD1CON1.B15 = 1;
}

//------------------------------------------------------------------------------